//-----------------------------------------------------------------------------
//
//	Merit Manager Unit
//
//-----------------------------------------------------------------------------
unit UFilterEnum;

interface

uses
	Windows, ActiveX, DirectShow9, SysUtils, Registry, Classes;

const
	IID_IPropertyBag: TGUID = '{55272A00-42CB-11CE-8135-00AA004BB851}';
  IID_ISpecifyPropertyPages: TGUID = '{B196B28B-BAB4-101A-B69C-00AA00341D07}';

  //Filter state flags
  fsHasName    = $01;
  fsProtected  = $02;
  fsFileExists = $04;

type
	TFilterItem = record
		Name:	  	String;
    FileName:	String;
    FileVersion: String;
    CLSID:		TGUID;
    Merit:		DWORD;
    Version:	DWORD;
    Flags:    DWORD;
  end;
  PFilterItem = ^TFilterItem;

//-----------------------------------------------------------------------------
//
//	TCategory Class
//
//-----------------------------------------------------------------------------

	TCategory = class(TList)
  private
  	fName: String;
    fCLSID:	TGUID;
    function GetItem(i: Integer): PFilterItem;
  public
		constructor Create(const aName: String; const aCLSID: TGUID);
    destructor Destroy; override;
    procedure Clear; override;
    procedure Enumerate;
    property Items[i: Integer]: PFilterItem read GetItem;
    property CLSID: TGUID read fCLSID;
    property Name: String read fName;
  end;

//-----------------------------------------------------------------------------
//
//	TCategoriesEnum Class
//
//-----------------------------------------------------------------------------
	TCategoriesEnum = class(TList)
  private
    fDirectShowItem: TCategory;
    function GetItem(i: Integer): TCategory;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear; override;
    procedure Enumerate;
    property Items[i: Integer]: TCategory read GetItem;
    property DirectShowItem: TCategory read fDirectShowItem;
  end;

procedure ExtractFilterInfo(var Filter: TFilterItem);
function WriteMerit(var Filter: TFilterItem): BOOL;
function DeleteFilterFromRegistry(const CLSID: String): Boolean;
function RegisterDLL(const FileName: String; Reg: Boolean): HRESULT;
function ValidFilename(const S: string): string;
function IsAdmin: Boolean;

implementation

var
  SystemPath: String;

//-----------------------------------------------------------------------------
//
//	TCategory Implementation
//
//-----------------------------------------------------------------------------

function SortFilters(Item1, Item2: Pointer): Integer;
begin
  result := AnsiCompareStr(PFilterItem(Item1).Name,PFilterItem(Item2).Name);
end;

function TCategory.GetItem(i: Integer): PFilterItem;
begin
	// make sure the parameter is valid
	result := NIL;
  if (i < 0) or (i >= Count) then exit;
  // return the requested item
  result := inherited Items[i];
end;

procedure TCategory.Enumerate;
var
  SysDevEnum: ICreateDevEnum;
	EnumCat: IEnumMoniker;
  Moniker: IMoniker;
  f: ULONG;
  hr: HRESULT;
  PropBag: IPropertyBag;
  varName: OleVariant;
  Item: TFilterItem;
  NewItem: PFilterItem;
  CLSID: String;
begin
	// clear what we may have
  Clear;
  try
  	// make sure everything works fine... we don't want any terrific crashes
  	try
    	// let's create the system device enumerator
     	hr := CoCreateInstance(CLSID_SystemDeviceEnum, NIL, CLSCTX_INPROC_SERVER,
								  IID_ICreateDevEnum, SysDevEnum);
      if FAILED(hr) or (SysDevEnum = NIL) then exit;
      // we want to enumerate categories
			hr := SysDevEnum.CreateClassEnumerator(fCLSID, EnumCat, 0);
      if FAILED(hr) or (EnumCat = NIL) then exit;
      // scan through the categories
      while (EnumCat.Next(1, Moniker, @F) = NOERROR) do
      begin
      	hr := Moniker.BindToStorage(NIL, NIL, IID_IPropertyBag, PropBag);
        if SUCCEEDED(hr) then
        begin
					// get the category name
          VariantInit(varName);
          PropBag.Read('FriendlyName', varName, NIL);
          Item.Name := String(varName);
          VariantClear(varName);
          // get the CLSID
          VariantInit(varName);
          PropBag.Read('CLSID', varName, NIL);
          CLSID := String(VarName);
          if CLSID <> '' then
          begin
            try
              Item.CLSID := StringToGuid(CLSID);
            except
              Item.CLSID := GUID_NULL;
            end;
            VariantClear(varName);
            // we're done with property bag
            PropBag := NIL;
            // if the category item is valid, we may add it to our list
            if not IsEqualGUID(Item.CLSID, GUID_NULL) then
            begin
              ExtractFilterInfo(Item);
              New(NewItem);
              NewItem^ := Item;
              Add(NewItem);
            end;
          end;
        end;
        Moniker:= NIL;
      end;
    except
    	MessageBox(0, 'Unexepcted Error in TCategory.Enumerate', 'Error', MB_ICONERROR);
    end;
  finally
    Sort(@SortFilters);
   	// clean all the interfaces
    PropBag:= NIL;
		Moniker:= NIL;
    EnumCat:= NIL;
    SysDevEnum:= NIL;
  end;
end;

procedure TCategory.Clear;
var
  i: Integer;
begin
  for i := 0 to Count-1 do
    Dispose(PFilterItem(Items[i]));
  inherited;
end;

constructor TCategory.Create(const aName: String; const aCLSID: TGUID);
begin
	inherited Create;
  fName := aName;
  fCLSID := aCLSID;
end;

destructor TCategory.Destroy;
begin
  Clear;
  inherited Destroy;
end;

//-----------------------------------------------------------------------------
//
//	TCategoriesEnum Implementation
//
//-----------------------------------------------------------------------------

function SortCategories(Item1, Item2: Pointer): Integer;
begin
  result := AnsiCompareStr(TCategory(Item1).Name,TCategory(Item2).Name);
end;

function TCategoriesEnum.GetItem(i: Integer): TCategory;
begin
	// make sure the parameter is valid
	result := NIL;
  if (i < 0) or (i >= Count) then exit;
  // return the requested item
  result := inherited Items[i];
end;

procedure TCategoriesEnum.Enumerate;
var
  SysDevEnum: ICreateDevEnum;
	EnumCat: IEnumMoniker;
  Moniker: IMoniker;
  f: ULONG;
  hr: HRESULT;
  PropBag: IPropertyBag;
  varName: OleVariant;
  CLSID: TGUID;
  Name: String;
begin
	// clear what we may have
  Clear;
  try
   	// make sure everything works fine... we don't want any terrific crashes
  	try
    	// let's create the system device enumerator
     	hr := CoCreateInstance(CLSID_SystemDeviceEnum, NIL, CLSCTX_INPROC_SERVER,
								  IID_ICreateDevEnum, SysDevEnum);
      if FAILED(hr) or (SysDevEnum = NIL) then exit;
      // we want to enumerate categories
			hr := SysDevEnum.CreateClassEnumerator(CLSID_ActiveMovieCategories, EnumCat, 0);
      if FAILED(hr) or (EnumCat = NIL) then exit;
      // scan through the categories
      while (EnumCat.Next(1, Moniker, @F) = NOERROR) do
      begin
       	hr := Moniker.BindToStorage(NIL, NIL, IID_IPropertyBag, PropBag);
        if SUCCEEDED(hr) then
        begin
					// get the category name
          VariantInit(varName);
          PropBag.Read('FriendlyName', varName, NIL);
          Name := String(varName);
          VariantClear(varName);
          // get the CLSID
          VariantInit(varName);
          PropBag.Read('CLSID', varName, NIL);
          try
           	CLSID:= StringToGuid(String(varName));
          except
           	CLSID := GUID_NULL;
          end;
          VariantClear(varName);
          // we're done with property bag
          PropBag := NIL;
          // if the category item is valid, we may add it to our list
          if not IsEqualGUID(CLSID, GUID_NULL) then
          begin
            Add(TCategory.Create(Name,CLSID));
            if (fDirectShowItem = NIL) and  (Pos('directshow',LowerCase(Name)) > 0) then
              fDirectShowItem := Items[Count-1];
          end;
        end;
        Moniker := NIL;
      end;
    except
     	MessageBox(0, 'Unexepcted Error in TCategoriesEnum.Enumerate', 'Error', MB_ICONERROR);
    end;
  finally
    Sort(@SortCategories);
  	// clean all the interfaces
    PropBag := NIL;
		Moniker:= NIL;
    EnumCat:= NIL;
    SysDevEnum:= NIL;
  end;
end;

procedure TCategoriesEnum.Clear;
var
  i: Integer;
begin
  for i := 0 to Count-1 do
    TCategory(Items[i]).Free;
  fDirectShowItem := nil;
  inherited;
end;

constructor TCategoriesEnum.Create;
begin
  inherited;
  Enumerate;
end;

destructor TCategoriesEnum.Destroy;
begin
  Clear;
  inherited Destroy;
end; 

//-----------------------------------------------------------------------------
//
//	Globals
//
//-----------------------------------------------------------------------------

function IsAdmin: Boolean;
const
  SECURITY_NT_AUTHORITY: TSIDIdentifierAuthority =(Value: (0, 0, 0, 0, 0, 5));
  SECURITY_BUILTIN_DOMAIN_RID = $00000020;
  DOMAIN_ALIAS_RID_ADMINS = $00000220;
var
  hAccessToken: THandle;
  ptgGroups: PTokenGroups;
  dwInfoBufferSize: DWORD;
  psidAdministrators: PSID;
  x: Integer;
  bSuccess: BOOL;
  Info: TOSVersionInfo;
begin
  Info.dwOSVersionInfoSize := SizeOf(Info);
  result := GetVersionEx(Info) and (Info.dwPlatformId <> VER_PLATFORM_WIN32_NT);
  if result then exit;
  bSuccess := OpenThreadToken(GetCurrentThread, TOKEN_QUERY, True, hAccessToken);
  if not bSuccess then
  begin
    if GetLastError = ERROR_NO_TOKEN then
    bSuccess := OpenProcessToken(GetCurrentProcess, TOKEN_QUERY, hAccessToken);
  end;
  if bSuccess then
  begin
    GetMem(ptgGroups, 1024);
    bSuccess := GetTokenInformation(hAccessToken, TokenGroups, ptgGroups, 1024, dwInfoBufferSize);
    CloseHandle(hAccessToken);
    if bSuccess then
    begin
      AllocateAndInitializeSid(SECURITY_NT_AUTHORITY, 2,
        SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_ADMINS,
        0, 0, 0, 0, 0, 0, psidAdministrators);
      for x := 0 to ptgGroups.GroupCount - 1 do
        if EqualSid(psidAdministrators, ptgGroups.Groups[x].Sid) then
        begin
          Result := True;
          Break;
        end;
      FreeSid(psidAdministrators);
    end;
    FreeMem(ptgGroups);
  end;
end;

function GetSystemPath: String;
var
  Buffer: array[0..MAX_PATH] of Char;
begin
  if GetSystemDirectory(@Buffer,SizeOf(Buffer)) = 0 then
    result := ''
  else
    result := IncludeTrailingPathDelimiter(Buffer);
end;

function ExpandEnvironment(const s: string): string;
var
  Buffer: array[0..MAX_PATH] of Char;
begin
  if ExpandEnvironmentStrings(PChar(s),Buffer,SizeOf(Buffer)) = 0 then
    Result := s
  else
    Result := Trim(Buffer);
end;

function ValidFilename(const S: string): string;
var
  i: Integer;
begin
  result := Trim(s);
  i := 1;
  while i <= Length(result) do
    if result[i] in [#0..#31,
                     '.', '/', '\', '*', '?',':', '"', #39, '<', '>', '|'] then
      result[i] := '_'
    else
      inc(i);
end;

function GetFileVersion(const FileName: string): string;
var
 VersionInfoSize, VersionInfoValueSize, Zero: DWord;
 VersionInfo, VersionInfoValue: Pointer;
begin
  Result := '';
  VersionInfoSize := GetFileVersionInfoSize(PChar(FileName), Zero);
  if VersionInfoSize = 0 then exit;
  GetMem(VersionInfo, VersionInfoSize);
  try
    if GetFileVersionInfo(PChar(FileName), 0, VersionInfoSize, VersionInfo) and
       VerQueryValue(VersionInfo, '\' { root block }, VersionInfoValue, VersionInfoValueSize) and
       (LongInt(VersionInfoValueSize) <> 0) then
     with TVSFixedFileInfo(VersionInfoValue^) do
     begin
       Result := IntToStr(HiWord(dwFileVersionMS)) + '.' +
                 IntToStr(LoWord(dwFileVersionMS)) + '.' +
                 IntToStr(HiWord(dwFileVersionLS)) + '.' +
                 IntToStr(LoWord(dwFileVersionLS));
     end;
  except
  end;
  FreeMem(VersionInfo);
end;

procedure ExtractFilterInfo(var Filter: TFilterItem);
var
  Reg: TRegistry;
	s: String;
  binaryData: record
               	Version: DWORD;
                Merit: DWORD;
                b: array[0..16384] of Byte;
              end;
begin
  Filter.Flags := 0;
  Filter.Merit := 0;
  Filter.Version := 0;
  Filter.FileName := '';
  Reg := TRegistry.Create;
  Reg.Access := KEY_READ; //Fixed: only reading is necessary
  // file name is located in HKEY_CLASSES_ROOT\CLSID\{filterCLSID}\InprocServer32
  Reg.RootKey:= HKEY_CLASSES_ROOT;
  try
    s := GuidToString(Filter.CLSID);
    if Reg.OpenKey('\CLSID\'+s+'\InprocServer32', FALSE) then
    // load the file name
    try
      Filter.Filename := Reg.ReadString('');
      if Filter.Filename <> '' then
      begin
        Filter.Flags := fsHasName;
        if ExtractFilePath(Filter.Filename) = '' then //no Path (XP: ksproxy.ax, Vista: additionally quartz.dll)
        begin
          Filter.Flags := Filter.Flags or fsProtected;
          Filter.Filename := SystemPath + Filter.Filename;
        end
        else if Filter.Filename[1] = '%' then //Vista: Movie Maker filters
          Filter.Filename := ExpandEnvironment(Filter.Filename)
        else if LowerCase(ExtractFilename(Filter.FileName)) = 'quartz.dll' then
          Filter.Flags := Filter.Flags or fsProtected;
        if FileExists(Filter.Filename) then
        begin
          Filter.Flags := Filter.Flags or fsFileExists;
          Filter.FileVersion := GetFileVersion(Filter.FileName);
        end;
      end;
    except
      //error accessing the registry
    end;
    // Merit and other stuff is located in
    // HKEY_CLASSES_ROOT\CLSID\{083863F1-70DE-11d0-BD40-00A0C911CE86}\Instance\{filterclsid}
    if Reg.OpenKeyReadOnly('\CLSID\{083863F1-70DE-11d0-BD40-00A0C911CE86}\Instance\' + s) then
    try
      Reg.ReadBinaryData('FilterData', binaryData, sizeof(binaryData));
      Filter.Version := binarydata.Version;
      Filter.Merit := binarydata.Merit;
    except
      //error accessing the registry
    end;
  except
    //error converting the GUID to a string
  end;
	Reg.Free;
end;

function DeleteFilterFromRegistry(const CLSID: String): Boolean;
var
  Reg: TRegistry;
begin
  result := false;
  if (CLSID = '') or (CLSID[1] <> '{') or (CLSID = '{00000000-0000-0000-0000-000000000000}') then
    exit;
  Reg := TRegistry.Create;
  Reg.RootKey := HKEY_CLASSES_ROOT;
  try
    result := Reg.DeleteKey('\CLSID\' + CLSID);
    result := Reg.DeleteKey('\CLSID\{083863F1-70DE-11d0-BD40-00A0C911CE86}\Instance\' + CLSID) or result;
    result := Reg.DeleteKey('\Filter\' + CLSID) or result;
  except
    result := false;
  end;
  Reg.Free;
end;

function WriteMerit(var Filter: TFilterItem): BOOL;
var
  Reg: TRegistry;
	s, s2: String;
  binaryData: record
               	Version: DWORD;
                Merit: DWORD;
                b: array[0..16384] of Byte;
              end;
  Size: Integer;
begin
  Reg:= TRegistry.Create;
  result := FALSE;
  try
  	try
      Reg.RootKey:= HKEY_CLASSES_ROOT;
      try
      	s := GuidToString(Filter.CLSID);
      except
      	s := GuidToString(GUID_NULL);
      end;
      // Merit and other stuff is located in
      // HKEY_CLASSES_ROOT\CLSID\{083863F1-70DE-11d0-BD40-00A0C911CE86}\Instance\{filterclsid}
      s2:= '\CLSID\{083863F1-70DE-11d0-BD40-00A0C911CE86}\Instance\';
      if Reg.OpenKey(s2 + s, FALSE) then
      begin
       	try
   				Size := Reg.ReadBinaryData('FilterData', binaryData, sizeof(binaryData));
          // apply new merit
          binaryData.Merit:= Filter.Merit;
          // write data back
          Reg.WriteBinaryData('FilterData', binaryData, size);
        except
				end;
      end;
      // HKEY_LOCAL_MACHINE\SOFTWARE\Classes\CLSID\{083863F1-70DE-11d0-BD40-00A0C911CE86}\Instance
      Reg.RootKey := HKEY_LOCAL_MACHINE;
      s2 := '\SOFTWARE\Classes\CLSID\{083863F1-70DE-11d0-BD40-00A0C911CE86}\Instance\';
      if Reg.OpenKey(s2 + s, FALSE) then
      begin
       	try
	     		Size := Reg.ReadBinaryData('FilterData', binaryData, sizeof(binaryData));
          // apply new merit
          binaryData.Merit := Filter.Merit;
          // write data back
          Reg.WriteBinaryData('FilterData', binaryData, size);
          result:= TRUE;
        except
				end;
      end;
    except
			// error accessing registry
    end;
  finally
  	Reg.Free;
  end;
end;

function RegisterDLL(const FileName: String; Reg: Boolean): HRESULT;
var
  RegProc: function: HRESULT; stdcall;
	Lib: Cardinal;
begin
	result := E_FAIL;
  Lib := LoadLibrary(PChar(FileName));
  if Lib <> 0 then
  try
    // try to register the ActiveX Server
    if Reg then
      RegProc := GetProcAddress(Lib, 'DllRegisterServer')
    else
      RegProc := GetProcAddress(Lib, 'DllUnregisterServer');
    if assigned(RegProc) then
      result := RegProc;
  finally
    FreeLibrary(Lib);
  end;
end;

initialization
  CoInitialize(NIL);
  SystemPath := GetSystemPath;
finalization
	CoUninitialize;
end.
