unit Unit1;

// 2003.11.03 V. 1.2 (Griga):
//- Merit Input Line added

// 2003.11.09 V 1.3 (by Griga):
// - Log tab & log file added
// - Problem with 120 dpi setting fixed

// 2003.12.29 V 1.4 (by Griga):
// - Button for opening a filter's property page added
// - Empty log file added, so that the Write Log File option is switched on as default

// 2007.05.23: V 1.5 (by Griga):
// - Fixed unnecessary registry write access, causing problems in limited user accounts
// - Added button for saving the the DirectShow filter list as text file.
// - DirectShow filters whose file is missing are greyed.
// - If the file is missing, the RFM offers to delete the according registry keys

// 2008.04.18: V 1.6 (by Griga)
// - Rewritten several parts by using faster and more flexible data containers (derived from TList)
// - Empty categories are not shown anymore
// - Added handling of environment variables in paths (Movie Maker filters under Vista)
// - Added handling of filenames without path (ksproxy.ax under XP/Vista, quartz.dll under Vista)
// - Essential filter files (those without path and additionally quartz.dll under XP)
//   cannot be unregistered anymore.
// - Displays a message box if admin rights are missing (e.g. for registering)
// - Added Refresh, Navigate and Save Category as Text File commands to popup menu
// - Added right-click selection of tree nodes
// - Added hints for some buttons
// - Added file version as filter subnode and in category text files
// - Larger default width/height
// - Faster memory management with FastMM4
// - Removed Blacklist Tab (we've enjoyed this joke long enough...)

interface

{$R WindowsXP.res}

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  StdCtrls, ComCtrls, Menus, ExtCtrls, Dialogs, UFilterEnum, ActiveX, DirectShow9,
  ShellAPI, ClipBrd;

type
  TMainForm = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    BClose: TButton;
    PopupMenu1: TPopupMenu;
    ItemUnregister: TMenuItem;
    N1: TMenuItem;
    TabSheet3: TTabSheet;
    InfoMemo: TMemo;
    Label2: TLabel;
    Image1: TImage;
    Label3: TLabel;
    Image2: TImage;
    BRegister: TButton;
    BUnregister: TButton;
    CBMerit: TComboBox;
    BSetValue: TButton;
    OpenD: TOpenDialog;
    ItemCopyCLSID: TMenuItem;
    TVFilters: TTreeView;
    BCopyGUID: TButton;
    BNavigate: TButton;
    ItemExpandAll: TMenuItem;
    ItemCollapseAll: TMenuItem;
    EMerit: TEdit;
    TabSheet4: TTabSheet;
    MLog: TMemo;
    CBLog: TCheckBox;
    BClearLog: TButton;
    BPropertyPage: TButton;
    Label4: TLabel;
    BSaveDSList: TButton;
    SaveD: TSaveDialog;
    ItemRefresh: TMenuItem;
    ItemNavigate: TMenuItem;
    ItemSaveCategory: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TVFiltersCustomDrawItem(Sender: TCustomTreeView;
      Node: TTreeNode; State: TCustomDrawState; var DefaultDraw: Boolean);
    procedure BCloseClick(Sender: TObject);
    procedure ItemExpandAllClick(Sender: TObject);
    procedure ItemCollapseAllClick(Sender: TObject);
    procedure TVFiltersChange(Sender: TObject; Node: TTreeNode);
    procedure BSetValueClick(Sender: TObject);
    procedure BNavigateClick(Sender: TObject);
    procedure BCopyGUIDClick(Sender: TObject);
    procedure BRegisterClick(Sender: TObject);
    procedure BUnregisterClick(Sender: TObject);
    procedure EKeyPress(Sender: TObject; var Key: Char);
    procedure CBMeritOnChange(Sender: TObject);
    procedure BClearClick(Sender: TObject);
    procedure BPropertyPageClick(Sender: TObject);
    procedure QueryEndMessage(var Message: TMessage); Message WM_QUERYENDSESSION;
    procedure BSaveDSListClick(Sender: TObject);
    procedure ItemRefreshClick(Sender: TObject);
    procedure TVFiltersMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ItemSaveCategoryClick(Sender: TObject);
  private
    { Private declarations }
    LogPath: String;
    Admin: Boolean;
    procedure Log(const s: String);
    procedure LogEnumFilters(const Action: String; Filename: String);
    function MyMessageBox(const Text: String; uType: DWord): Boolean;
    procedure SaveList(Cat: TCategory);
  public
    { Public declarations }
    Categories: TCategoriesEnum;
    CurrentFilter: TTreeNode;
    procedure BuildTree;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

const
  MeritPreText = 'Merit : 0x';
  CLSIDPreText = 'CLSID : ';
  FilePreText = 'File Name : ';
  FileVersionPreText = 'File Version : ';
  VersionPreText = 'Version : ';
  LogName = 'FilterManager.log';


function TMainForm.MyMessageBox(const Text: String; uType: DWord): Boolean;
begin
  result := MessageBox(Handle,PChar(Text),'RadLight Filter Manager',uType) = IDOK;
end;

//-----------------------------------------------------------------------------
//
//	Log File Handling
//
//-----------------------------------------------------------------------------

procedure TMainForm.Log(const s: String);
begin
  with MLog.Lines do
  begin
    if Count = 0 then
      Add('*** RadLight Filter Manager Log File ***');
    Add('');
    Add(DateTimeToStr(Now));
    Add(s);
  end;
end;

procedure TMainForm.LogEnumFilters(const Action: String; Filename: String);
// Log all filters that are contained in the file
var
  i,j: Integer;
begin
  Log(Action + Filename);
  Filename := LowerCase(Filename);
  for i := 0 to Categories.Count-1 do
    with Categories.Items[i] do
      for j := 0 to Count-1 do
        if Filename = LowerCase(Items[j].FileName) then
          MLog.Lines.Add(Items[j].Name);
end;

procedure TMainForm.BClearClick(Sender: TObject);
begin
  MLog.Lines.Clear;
end;

//-----------------------------------------------------------------------------
//
//	OnClick Handlers
//
//-----------------------------------------------------------------------------

procedure TMainForm.BRegisterClick(Sender: TObject);
var
  hr: HRESULT;
	i: Integer;
  s: String;
begin
  if not Admin then
  begin
    MyMessageBox('Registering filters requires administrator rights!',MB_ICONERROR);
    exit;
  end;
	if OpenD.Execute then
  begin
  	for i := 0 to OpenD.Files.Count-1 do
    begin
			hr := RegisterDLL(OpenD.Files.Strings[i],true);
      if SUCCEEDED(hr) then
      begin
        // refresh the filter tree
        BuildTree;
        LogEnumFilters('Registered: ',OpenD.Files.Strings[i]);
       	s := 'DllRegisterServer succeeded in '+OpenD.Files.Strings[i];
				MyMessageBox(s, MB_ICONINFORMATION);
      end
      else begin
       	s := 'DllRegisterServer failed in '+OpenD.Files.Strings[i]+' with error code : 0x';
        s := s + IntToHex(hr, 8);
				MyMessageBox(s, MB_ICONERROR);
      end;
    end;
  end;
end;

procedure TMainForm.BUnregisterClick(Sender: TObject);
var
	hr: HRESULT;
  Item: PFilterItem;
begin
	if (CurrentFilter = NIL) or (CurrentFilter.Data = NIL) then exit;
  if not Admin then
  begin
    MyMessageBox('Unregistering filters requires administrator rights!',MB_ICONERROR);
    exit;
  end;
  try
    Item := CurrentFilter.Data;
    hr := RegisterDLL(Item.FileName,false);
    if SUCCEEDED(hr) then
    begin
      LogEnumFilters('Unregistered: ',Item.FileName);
			MyMessageBox('DllUnregisterServer succeeded in ' + Item.FileName, MB_ICONINFORMATION);
    end
    else if Item.Flags and fsFileExists = 0 then //file does not exist
    begin
      if MyMessageBox('Cannot unregister - file is missing. Delete the registry entries?',
                      MB_OKCANCEL or MB_ICONQUESTION) then
      begin
        hr := S_OK;
        if DeleteFilterFromRegistry(GuidToString(Item.CLSID)) then
          LogEnumFilters('Removed registry keys: ',Item.FileName)
        else begin
          MyMessageBox('Deleting the registry entries failed', MB_ICONERROR);
          hr := E_FAIL;
        end;
      end
    end
    else
      MyMessageBox('DllUnregisterServer failed in '+ Item.FileName +' with error code : 0x' + IntToHex(hr, 8), MB_ICONERROR);
  except
    hr := E_FAIL;
  end;
  // refresh the filter tree
  if SUCCEEDED(hr) then
    BuildTree;
end;

procedure TMainForm.BNavigateClick(Sender: TObject);
begin
	if (CurrentFilter = NIL) or (CurrentFilter.Data = NIL) then exit;
	// open the explorer in the filter directory
  try
    ShellExecute(Handle, 'open', PChar(ExtractFilePath(PFilterItem(CurrentFilter.Data).Filename)), '', '', SW_SHOWNORMAL);
  except
  end;
end;

procedure TMainForm.BCopyGUIDClick(Sender: TObject);
begin
	if (CurrentFilter = NIL) or (CurrentFilter.Data = NIL) then exit;
	// put the required CLSID into clipboard
  try
    ClipBoard.SetTextBuf(PChar(GuidToString(PFilterItem(CurrentFilter.Data).CLSID)));
  except
  end;
end;

procedure TMainForm.BSetValueClick(Sender: TObject);
var
  Item: PFilterItem;
  OldMerit: DWord;
begin
	if (CurrentFilter = NIL) or (CurrentFilter.Data = NIL) then exit;
  if not Admin then
  begin
    MyMessageBox('Setting merit values requires administrator rights!',MB_ICONERROR);
    exit;
  end;
  try
    Item := PFilterItem(CurrentFilter.Data);
    OldMerit := Item.Merit;
    // set the new merit
    Item.Merit := StrToInt('$' + EMerit.Text);
    if Item.Merit = OldMerit then exit;
    if WriteMerit(Item^) then
    begin
      Log('Changed merit: ' + CurrentFilter.Text);
      MLog.Lines.Add(IntToHex(OldMerit,8) + ' --> ' + EMerit.Text);
      CurrentFilter.Item[3].Text := MeritPreText + EMerit.Text;
    end
    else begin
      Item.Merit := OldMerit;
      MyMessageBox('Cannot update merit', MB_ICONERROR);
    end;
  except
  	// we don't care..
  end;
end;

procedure TMainForm.BPropertyPageClick(Sender: TObject);
var
  Filter: IBaseFilter;
  pSpecify: ISpecifyPropertyPages;
  caGuid: tagCAUUID;
begin
	if (CurrentFilter = NIL) or (CurrentFilter.Data = NIL) then exit;
  try
    if SUCCEEDED(CoCreateInstance(PFilterItem(CurrentFilter.Data).CLSID,NIL,CLSCTX_INPROC_SERVER,
           IID_IBaseFilter,Filter)) and
       SUCCEEDED(Filter.QueryInterface(IID_ISpecifyPropertyPages,
           pSpecify)) then
    begin
      pSpecify.GetPages(caGuid);
      if caGuid.cElems > 0 then
      begin
        OleCreatePropertyFrame(
         Handle, // Parent window
         0, // x (Reserved)
         0, // y (Reserved)
         PWideChar(WideString(CurrentFilter.Text)), // Caption for the dialog box
         1, // Number of filters
         @Filter, // Pointer to the filter
         caGUID.cElems, // Number of property pages
         caGUID.pElems, // Pointer to property page CLSIDs
         0, // Locale identifier
         0, // Reserved
         NIL // Reserved
        );
        CoTaskMemFree(caGUID.pElems);
      end;
    end;
  except
  end;
end;

procedure TMainForm.SaveList(Cat: TCategory);
const
  Delimiter = ','; //comma separated
var
  List: TStringList;
  Item: PFilterItem;
  i: Integer;
begin
  if Cat = nil then exit;
  SaveD.FileName := ValidFilename(Cat.Name) + '.txt';
  if not SaveD.Execute then exit;
  //Now list all DS filters
  List := TStringList.Create;
  try
    List.Add('Filter' + Delimiter +
             'CLSID' + Delimiter +
             'Merit' + Delimiter +
             'Path' + Delimiter +
             'FileVersion');
    for i := 0 to Cat.Count-1 do
    begin
      Item := Cat.Items[i];
      if Item <> NIL then
      try
        List.Add('"' + Item.Name + '"' + Delimiter + GuidToString(Item.CLSID) + Delimiter +
                 IntToHex(Item.Merit,8) + Delimiter + '"' + Item.Filename + '"' + Delimiter +
                 Item.FileVersion);
      except
      end;
    end;
    List.SaveToFile(SaveD.FileName);
  except
  end;
  List.Free;
end;

procedure TMainForm.BSaveDSListClick(Sender: TObject);
begin
  SaveList(Categories.DirectShowItem);
end;

procedure TMainForm.ItemSaveCategoryClick(Sender: TObject);
var
  Node: TTreeNode;
begin
  Node := TVFilters.Selected;
  if Node <> nil then
  begin
    while Node.Level > 0 do
      Node := Node.Parent;
    SaveList(Node.Data);
  end;
end;

procedure TMainForm.BCloseClick(Sender: TObject);
begin
	Close;
end;

procedure TMainForm.ItemExpandAllClick(Sender: TObject);
begin
	TVFilters.FullExpand;
end;

procedure TMainForm.ItemCollapseAllClick(Sender: TObject);
begin
	TVFilters.FullCollapse;
end;

procedure TMainForm.ItemRefreshClick(Sender: TObject);
begin
  BuildTree;
end;

//-----------------------------------------------------------------------------
//
//	Building and Drawing the Tree
//
//-----------------------------------------------------------------------------

procedure TMainForm.BuildTree;
var
  Node, SubNode: TTreeNode;
	i, j: Integer;
  Cat: TCategory;
  Filt: PFilterItem;
  s: String;
begin
  TVFilters.Items.BeginUpdate;
  try
    TVFilters.Items.Clear;
    // scan through the categories
    for i := 0 to Categories.Count - 1 do
    begin
      Cat := Categories.Items[i];
      if Cat = NIL then continue;
      Cat.Enumerate;
      if Cat.Count = 0 then continue;
      Node := TVFilters.Items.AddObject(NIL, Cat.Name, Cat);
      for j := 0 to Cat.Count-1 do
      begin
        Filt := Cat.Items[j];
        if Filt = NIL then continue;
        SubNode := TVFilters.Items.AddChildObject(Node, Filt.Name,Filt);
        // add filter information :
        // FileName
        TVFilters.Items.AddChildObject(SubNode, FilePreText+Filt.FileName,Filt);
        //File version
        TVFilters.Items.AddChildObject(SubNode, FileVersionPreText+Filt.FileVersion,Filt);
        // GUID
        try
          s := GuidToString(Filt.CLSID);
        except
          s := 'INVALID GUID';
        end;
        TVFilters.Items.AddChild(SubNode, CLSIDPreText+s);
        // Merit
        TVFilters.Items.AddChild(SubNode, MeritPreText+IntToHex(Filt.Merit, 8));
        // Version
        TVFilters.Items.AddChild(SubNode, VersionPreText+IntToStr(Filt.Version));
      end;
    end;
	finally
    TVFilters.Items.EndUpdate;
  end;
end;

procedure TMainForm.TVFiltersCustomDrawItem(Sender: TCustomTreeView;
  Node: TTreeNode; State: TCustomDrawState; var DefaultDraw: Boolean);
var
  Item: PFilterItem;
begin
  Item := Node.Data;
	with TVFilters.Canvas do
  begin
    case Node.Level of
      0: // drawing a category
        Font.Color:= clBlack;
      1: // drawing a filter
        if (Item <> NIL) and (Item.Flags and fsFileExists <> 0) then //file exists
          Font.Color:= clNavy
        else
          Font.Color := clGray;
      2: // drawing a information
        if (Item = NIL) or (Item.Flags and fsFileExists <> 0) then
          Font.Color:= clRed
        else
          Font.Color := clGray;
    end;
    // selected color
    if cdsSelected in State then
    begin
 			Brush.Color:= RGB(200, 220, 255);
      Pen.Color:= RGB(200, 220, 255);
    end;
  end;
end;

//-----------------------------------------------------------------------------
//
//	OnChange, Mouse and Keyboard Events
//
//-----------------------------------------------------------------------------

procedure TMainForm.TVFiltersChange(Sender: TObject; Node: TTreeNode);
var
  Item: PFilterItem;
begin
 	CurrentFilter := NIL;
  Item := NIL;
  if Node <> NIL then
  begin
    case Node.Level of
      //0: category selected... - no filter
      1: // Filters are Level 1
        CurrentFilter := Node;
      2: // information node selected -> find its parent
        CurrentFilter := Node.Parent;
    end;
    if CurrentFilter <> NIL then
      Item := CurrentFilter.Data;
  end;
  // enable / disable button
  BCopyGUID.Enabled := Item <> NIL;
  ItemCopyCLSID.Enabled := BCopyGUID.Enabled;
  BPropertyPage.Enabled := BCopyGUID.Enabled and
    (Item.Flags and fsFileExists <> 0);
  BSetValue.Enabled := BCopyGUID.Enabled;
  EMerit.Enabled := BCopyGUID.Enabled;
  if EMerit.Enabled then  //Transfer the merit to the input line
    EMerit.Text := IntToHex(Item.Merit,8);

  BUnregister.Enabled := BCopyGUID.Enabled and
    (Item.Flags and fsProtected = 0);
  ItemUnregister.Enabled := BUnregister.Enabled;
  BNavigate.Enabled := BCopyGUID.Enabled and (Item.FileName <> '');
  ItemNavigate.Enabled := BNavigate.Enabled;
end;

procedure TMainForm.CBMeritOnChange(Sender: TObject);
var
  NewMerit: Cardinal;
begin
  // set the new merit
	case (CBMerit.ItemIndex) of
  	0: NewMerit:= MERIT_PREFERRED + 255;
    1: NewMerit:= MERIT_PREFERRED + 1;
    2: NewMerit:= MERIT_PREFERRED;
    3: NewMerit:= MERIT_NORMAL + 1;
    4: NewMerit:= MERIT_NORMAL;
    5: NewMerit:= MERIT_UNLIKELY;
    6: NewMerit:= MERIT_DO_NOT_USE;
    7: NewMerit:= MERIT_SW_COMPRESSOR;
    8: NewMerit:= MERIT_HW_COMPRESSOR;
    else NewMerit:= MERIT_UNLIKELY;	// in case something went wrong.. you never know..
  end;
  EMerit.Text:=IntToHex(NewMerit, 8);
end;

procedure TMainForm.EKeyPress(Sender: TObject; var Key: Char);
//keys for hex input
begin
  if not (Key in [#1..#31,'0'..'9','A'..'F','a'..'f']) then key := #0;
end;

procedure TMainForm.TVFiltersMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  Node: TTreeNode;
  P: TPoint;
begin
  if Button = mbRight then
  begin
    Node := TVFilters.GetNodeAt(X,Y);
    if assigned(Node) then
      TVFilters.Selected := Node;
    P := TVFilters.ClientToScreen(Point(X,Y));
    PopupMenu1.Popup(P.X,P.Y);
  end;
end;

//-----------------------------------------------------------------------------
//
//	Form Initializing & Uninitializing
//
//-----------------------------------------------------------------------------

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Admin := IsAdmin;
  LogPath := ExtractFilePath(ParamStr(0))+LogName;
  CBLog.Checked := FileExists(LogPath);
  if CBLog.Checked then
  try
    MLog.Lines.LoadFromFile(LogPath);
  except
  end;
	Categories := TCategoriesEnum.Create;
  BuildTree;
end;

procedure TMainForm.QueryEndMessage(var Message: TMessage);
begin
  try 
    if CBLog.Checked then
      MLog.Lines.SaveToFile(LogPath)
    else
      DeleteFile(LogPath);
  except
  end;
  Message.Result := 1;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  try
    if CBLog.checked then
      MLog.Lines.SaveToFile(LogPath)
    else
      DeleteFile(LogPath);
  except
  end;
	Categories.Free;
end;

end.
